@@ -5,9 +5,11 @@ from astropy.extern.six.moves import zip
 from astropy.extern.six.moves import range as xrange
 from astropy.wcs import WCS
 from astropy.utils.console import ProgressBar
+import warnings
 
+from .utils import BadVelocitiesWarning
 from .cube_utils import _map_context
-from .lower_dimensional_structures import OneDSpectrum
+from .lower_dimensional_structures import VaryingResolutionOneDSpectrum, OneDSpectrum
 from .spectral_cube import VaryingResolutionSpectralCube
 
 
@@ -87,7 +89,7 @@ def _fourier_shifter(x, shift, axis):
     # m_shape = [1] * x.ndim
     # m_shape[axis] = m.shape[0]
     # m = m.reshape(m_shape)
-    slices = [slice(None) if ii == axis else None for ii in range(x.ndim)]
+    slices = tuple([slice(None) if ii == axis else None for ii in range(x.ndim)])
     m = m[slices]
     phase = np.exp(-2 * np.pi * m * 1j * shift)
     x2 = np.real(np.fft.ifft(ftx * phase, axis=axis))
@@ -225,6 +227,18 @@ def stack_spectra(cube, velocity_surface, v0=None,
     if not np.isclose(vdiff2.value, vdiff.value, rtol=vdiff_tol):
         raise ValueError("Cannot shift spectra on a non-linear axes")
 
+    vmax = cube.spectral_axis.to(vel_unit).max()
+    vmin = cube.spectral_axis.to(vel_unit).min()
+
+    if ((np.any(velocity_surface > vmax) or
+         np.any(velocity_surface < vmin))):
+        warnings.warn("Some velocities are outside the allowed range and will be "
+                      "masked out.", BadVelocitiesWarning)
+        velocity_surface = u.Quantity(np.where(
+            (velocity_surface < vmax) &
+            (velocity_surface > vmin),
+            velocity_surface, np.nan), velocity_surface.unit)
+
     pix_shifts = vdiff_sign * ((velocity_surface.to(vel_unit) -
                                 v0.to(vel_unit)) / vdiff).value[xy_posns]
 
@@ -288,11 +302,17 @@ def stack_spectra(cube, velocity_surface, v0=None,
 
     stacked = stack_function(shifted_spectra_array, axis=0)
 
-    stack_spec = \
-        OneDSpectrum(stacked, unit=cube.unit, wcs=WCS(new_header),
-                     header=new_header,
-                     meta=cube.meta, spectral_unit=vel_unit,
-                     beams=cube.beams if hasattr(cube, "beams") else None)
+    if hasattr(cube, 'beams'):
+        stack_spec = VaryingResolutionOneDSpectrum(stacked, unit=cube.unit,
+                                                   wcs=WCS(new_header),
+                                                   header=new_header,
+                                                   meta=cube.meta,
+                                                   spectral_unit=vel_unit,
+                                                   beams=cube.beams)
+    else:
+        stack_spec = OneDSpectrum(stacked, unit=cube.unit, wcs=WCS(new_header),
+                                  header=new_header, meta=cube.meta,
+                                  spectral_unit=vel_unit, beam=cube.beam)
 
     return stack_spec
 
