@@ -9,51 +9,31 @@ from astropy import convolution
 from astropy import units as u
 from astropy import wcs
 #from astropy import log
-from astropy.io.fits import Header, Card, HDUList, PrimaryHDU
-from radio_beam import Beam
+from astropy.io.fits import Header, HDUList, PrimaryHDU, BinTableHDU, FITS_rec
+from radio_beam import Beam, Beams
 
 from .io.core import determine_format
 from . import spectral_axis
-from .utils import SliceWarning
+from .utils import SliceWarning, BeamWarning, SmoothingWarning, FITSWarning
 from .cube_utils import convert_bunit
 from . import wcs_utils
 from .masks import BooleanArrayMask, MaskBase
 
 from .base_class import (BaseNDClass, SpectralAxisMixinClass,
                          SpatialCoordMixinClass, MaskableArrayMixinClass,
-                         MultiBeamMixinClass
+                         MultiBeamMixinClass, BeamMixinClass,
+                         HeaderMixinClass
                         )
 from . import cube_utils
 
 __all__ = ['LowerDimensionalObject', 'Projection', 'Slice', 'OneDSpectrum']
 
 
-class LowerDimensionalObject(u.Quantity, BaseNDClass):
+class LowerDimensionalObject(u.Quantity, BaseNDClass, HeaderMixinClass):
     """
     Generic class for 1D and 2D objects.
     """
 
-    @property
-    def header(self):
-        header = self._header
-        # This inplace update is OK; it's not bad to overwrite WCS in this
-        # header
-        if self.wcs is not None:
-            header.update(self.wcs.to_header())
-        header['BUNIT'] = self.unit.to_string(format='fits')
-        for keyword in header:
-            if 'NAXIS' in keyword:
-                del header[keyword]
-        header.insert(2, Card(keyword='NAXIS', value=self.ndim))
-        for ind,sh in enumerate(self.shape[::-1]):
-            header.insert(3+ind, Card(keyword='NAXIS{0:1d}'.format(ind+1),
-                                      value=sh))
-
-        if 'beam' in self.meta:
-            header.update(self.meta['beam'].to_header_keywords())
-
-        return header
-
     @property
     def hdu(self):
         if self.wcs is None:
@@ -83,10 +63,20 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass):
         if format is None:
             format = determine_format(filename)
         if format == 'fits':
-            try:
-                self.hdu.writeto(filename, overwrite=overwrite)
-            except TypeError:
-                self.hdu.writeto(filename, clobber=overwrite)
+            # Spectra may have HDUList objects instead of HDUs because they
+            # have a beam table attached, so we want to try that first
+            # (a more elegant way to write this might be to do "self._hdu_general.write"
+            # and create a property `self._hdu_general` that selects the right one...)
+            if hasattr(self, 'hdulist'):
+                try:
+                    self.hdulist.writeto(filename, overwrite=overwrite)
+                except TypeError:
+                    self.hdulist.writeto(filename, clobber=overwrite)
+            elif hasattr(self, 'hdu'):
+                try:
+                    self.hdu.writeto(filename, overwrite=overwrite)
+                except TypeError:
+                    self.hdu.writeto(filename, clobber=overwrite)
         else:
             raise ValueError("Unknown format '{0}' - the only available "
                              "format at this time is 'fits'")
@@ -147,13 +137,19 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass):
         return new
 
     def __array_finalize__(self, obj):
-        #log.debug("Finalizing self={0}{1} obj={2}{3}"
-        #          .format(self, type(self), obj, type(obj)))
         self._wcs = getattr(obj, '_wcs', None)
         self._meta = getattr(obj, '_meta', None)
         self._mask = getattr(obj, '_mask', None)
         self._header = getattr(obj, '_header', None)
         self._spectral_unit = getattr(obj, '_spectral_unit', None)
+        self._fill_value = getattr(obj, '_fill_value', np.nan)
+        self._wcs_tolerance = getattr(obj, '_wcs_tolerance', 0.0)
+
+        if isinstance(obj, VaryingResolutionOneDSpectrum):
+            self._beams = getattr(obj, '_beams', None)
+        else:
+            self._beam = getattr(obj, '_beam', None)
+
         super(LowerDimensionalObject, self).__array_finalize__(obj)
 
     @property
@@ -168,6 +164,12 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass):
         """
         return np.asarray(self)
 
+    @property
+    def _data(self):
+        # the _data property is required by several other mixins
+        # (which probably means defining it here is a bad design)
+        return self.array
+
     @property
     def quantity(self):
         """
@@ -268,11 +270,41 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass):
             self._mask = nomask
         return self
 
-class Projection(LowerDimensionalObject, SpatialCoordMixinClass):
+    def _initial_set_mask(self, mask):
+        """
+        Helper tool to validate mask when originally setting it in __new__
+
+        Note that because this is intended to be used in __new__, order
+        matters: ``self`` must have ``_wcs``, for example.
+        """
+        if mask is None:
+            mask = BooleanArrayMask(np.ones_like(self.value, dtype=bool),
+                                    self._wcs, shape=self.value.shape)
+        elif isinstance(mask, np.ndarray):
+            if mask.shape != self.value.shape:
+                raise ValueError("Mask shape must match the {0} shape."
+                                 .format(self.__class__.__name__)
+                                )
+            mask = BooleanArrayMask(mask, self._wcs, shape=self.value.shape)
+        elif isinstance(mask, MaskBase):
+            pass
+        else:
+            raise TypeError("mask of type {} is not a supported mask "
+                            "type.".format(type(mask)))
+
+        # Validate the mask before setting
+        mask._validate_wcs(new_data=self.value, new_wcs=self._wcs,
+                           wcs_tolerance=self._wcs_tolerance)
+
+        self._mask = mask
+
+
+class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
+                 MaskableArrayMixinClass, BeamMixinClass):
 
     def __new__(cls, value, unit=None, dtype=None, copy=True, wcs=None,
                 meta=None, mask=None, header=None, beam=None,
-                read_beam=False):
+                fill_value=np.nan, read_beam=False, wcs_tolerance=0.0):
 
         if np.asarray(value).ndim != 2:
             raise ValueError("value should be a 2-d array")
@@ -284,7 +316,11 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass):
                                   copy=copy).view(cls)
         self._wcs = wcs
         self._meta = {} if meta is None else meta
-        self._mask = mask
+        self._wcs_tolerance = wcs_tolerance
+
+        self._initial_set_mask(mask)
+
+        self._fill_value = fill_value
         if header is not None:
             self._header = header
         else:
@@ -296,7 +332,9 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass):
             elif read_beam:
                 beam = cube_utils.try_load_beam(header)
                 if beam is None:
-                    warnings.warn("Cannot load beam from header.")
+                    warnings.warn("Cannot load beam from header.",
+                                  BeamWarning
+                                 )
 
         if beam is not None:
             self.beam = beam
@@ -320,23 +358,70 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass):
         meta = self.meta.copy()
         meta['beam'] = beam
 
-        self = Projection(self.value, unit=self.unit, wcs=self.wcs,
-                          meta=meta, header=self.header,
-                          beam=beam)
+        return self._new_projection_with(beam=beam, meta=meta)
 
-        return self
+    def with_fill_value(self, fill_value):
+        """
+        Create a new :class:`Projection` or :class:`Slice` with a different
+        ``fill_value``.
+        """
+        return self._new_projection_with(fill_value=fill_value)
 
     @property
-    def beam(self):
-        return self._beam
+    def _new_thing_with(self):
+        return self._new_projection_with
 
-    @beam.setter
-    def beam(self, obj):
+    def _new_projection_with(self, data=None, wcs=None, mask=None, meta=None,
+                             fill_value=None, spectral_unit=None, unit=None,
+                             header=None, wcs_tolerance=None, beam=None,
+                             **kwargs):
 
-        if not isinstance(obj, Beam):
-            raise TypeError("beam must be a radio_beam.Beam object.")
+        data = self._data if data is None else data
+        if unit is None and hasattr(data, 'unit'):
+            if data.unit != self.unit:
+                raise u.UnitsError("New data unit '{0}' does not"
+                                   " match unit '{1}'.  You can"
+                                   " override this by specifying the"
+                                   " `unit` keyword."
+                                   .format(data.unit, self.unit))
+            unit = data.unit
+        elif unit is None:
+            unit = self.unit
+        elif unit is not None:
+            # convert string units to Units
+            if not isinstance(unit, u.Unit):
+                unit = u.Unit(unit)
 
-        self._beam = obj
+            if hasattr(data, 'unit'):
+                if u.Unit(unit) != data.unit:
+                    raise u.UnitsError("The specified new cube unit '{0}' "
+                                       "does not match the input unit '{1}'."
+                                       .format(unit, data.unit))
+            else:
+                data = u.Quantity(data, unit=unit, copy=False)
+
+        wcs = self._wcs if wcs is None else wcs
+        mask = self._mask if mask is None else mask
+        if meta is None:
+            meta = {}
+            meta.update(self._meta)
+        if unit is not None:
+            meta['BUNIT'] = unit.to_string(format='FITS')
+
+        fill_value = self._fill_value if fill_value is None else fill_value
+
+        if beam is None:
+            if hasattr(self, 'beam'):
+                beam = self.beam
+
+        newproj = self.__class__(value=data, wcs=wcs, mask=mask, meta=meta,
+                                 unit=unit, fill_value=fill_value,
+                                 header=header or self._header,
+                                 wcs_tolerance=wcs_tolerance or self._wcs_tolerance,
+                                 beam=beam,
+                                 **kwargs)
+
+        return newproj
 
     @staticmethod
     def from_hdu(hdu):
@@ -562,12 +647,15 @@ class Slice(Projection):
     pass
 
 
-class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
-                   SpectralAxisMixinClass):
+class BaseOneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
+                       SpectralAxisMixinClass):
+    """
+    Properties shared between OneDSpectrum and VaryingResolutionOneDSpectrum.
+    """
 
     def __new__(cls, value, unit=None, dtype=None, copy=True, wcs=None,
                 meta=None, mask=None, header=None, spectral_unit=None,
-                fill_value=np.nan, beams=None, wcs_tolerance=0.0):
+                fill_value=np.nan, wcs_tolerance=0.0):
 
         #log.debug("Creating a OneDSpectrum with __new__")
 
@@ -583,24 +671,8 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         self._meta = {} if meta is None else meta
         self._wcs_tolerance = wcs_tolerance
 
-        if mask is None:
-            mask = BooleanArrayMask(np.ones_like(self.value, dtype=bool),
-                                    self._wcs, shape=self.value.shape)
-        elif isinstance(mask, np.ndarray):
-            if mask.shape != self.value.shape:
-                raise ValueError("Mask shape must match the spectrum shape.")
-            mask = BooleanArrayMask(mask, self._wcs, shape=self.value.shape)
-        elif isinstance(mask, MaskBase):
-            pass
-        else:
-            raise TypeError("mask of type {} is not a supported mask "
-                            "type.".format(type(mask)))
+        self._initial_set_mask(mask)
 
-        # Validate the mask before setting
-        mask._validate_wcs(new_data=self.value, new_wcs=self._wcs,
-                           wcs_tolerance=self._wcs_tolerance)
-
-        self._mask = mask
         self._fill_value = fill_value
         if header is not None:
             self._header = header
@@ -615,13 +687,6 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
             elif self._wcs is not None:
                 self._spectral_unit = u.Unit(self._wcs.wcs.cunit[0])
 
-        if beams is not None:
-            self.beams = beams
-
-        # HACK: OneDSpectrum should eventually become not-a-quantity
-        # Maybe it should be a u.Quantity(np.ma)?
-        self._data = self.value
-
         return self
 
     def __repr__(self):
@@ -655,25 +720,29 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         else:
             unit = None
 
-        beams = cube_utils.try_load_beams(hdul)
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', category=FITSWarning)
+            beam = cube_utils.try_load_beams(hdul)
+            if hasattr(beam, '__len__'):
+                beams = beam
+            else:
+                beams = None
 
-        self = OneDSpectrum(hdu.data, unit=unit, wcs=mywcs, meta=meta,
-                            header=hdu.header, beams=beams)
+        if beams is not None:
+            self = VaryingResolutionOneDSpectrum(hdu.data, unit=unit,
+                                                 wcs=mywcs, meta=meta,
+                                                 header=hdu.header,
+                                                 beams=beams)
+        else:
+            beam = cube_utils.try_load_beam(hdu.header)
+            self = OneDSpectrum(hdu.data, unit=unit, wcs=mywcs, meta=meta,
+                                header=hdu.header, beam=beam)
 
         return self
 
     @property
     def header(self):
-        header = self._header
-        # This inplace update is OK; it's not bad to overwrite WCS in this
-        # header
-        if self.wcs is not None:
-            header.update(self.wcs.to_header())
-        header['BUNIT'] = self.unit.to_string(format='fits')
-        header.insert(2, Card(keyword='NAXIS', value=self.ndim))
-        for ind,sh in enumerate(self.shape[::-1]):
-            header.insert(3+ind, Card(keyword='NAXIS{0:1d}'.format(ind+1),
-                                      value=sh))
+        header = super(BaseOneDSpectrum, self).header
 
         # Preserve the spectrum's spectral units
         if 'CUNIT1' in header and self._spectral_unit != u.Unit(header['CUNIT1']):
@@ -684,7 +753,6 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
 
         return header
 
-
     @property
     def spectral_axis(self):
         """
@@ -723,7 +791,6 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         if filename is not None:
             pyplot.gcf().savefig(filename)
 
-
     def with_spectral_unit(self, unit, velocity_convention=None,
                            rest_value=None):
 
@@ -750,12 +817,15 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
                                        header=newheader)
 
     def __getitem__(self, key, **kwargs):
+        # Ideally, this could just be in VaryingResolutionOneDSpectrum,
+        # but it's about the code is about the same length by just
+        # keeping it here.
         try:
-            beams = self.beams[key]
-        except (AttributeError,TypeError):
-            beams = None
+            kwargs['beams'] = self.beams[key]
+        except (AttributeError, TypeError):
+            pass
 
-        new_qty = super(OneDSpectrum, self).__getitem__(key, beams=beams)
+        new_qty = super(BaseOneDSpectrum, self).__getitem__(key)
 
         if isinstance(key, slice):
 
@@ -781,22 +851,17 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
                 new_qty[bad] = np.nan
             return new_qty
 
-    @property
-    def hdu(self):
-        if hasattr(self, 'beams'):
-            warnings.warn("There are multiple beams for this spectrum that "
-                          "are being ignored when creating the HDU.")
-        return super(OneDSpectrum, self).hdu
-
-    @property
-    def hdulist(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            hdu = self.hdu
-
-        beamhdu = cube_utils.beams_to_bintable(self.beams)
-
-        return HDUList([hdu, beamhdu])
+    def __getattribute__(self, attrname):
+        # This is a hack to handle dimensionality-reducing functions
+        # We want spectrum.max() to return a Quantity, not a spectrum
+        # Long-term, we really want `OneDSpectrum` to not inherit from
+        # `Quantity`, but for now this approach works.... we just have
+        # to add more functions to this list.
+        if attrname in ('min', 'max', 'std', 'mean', 'sum', 'cumsum',
+                        'nansum', 'ptp', 'var'):
+            return getattr(self.quantity, attrname)
+        else:
+            return super(BaseOneDSpectrum, self).__getattribute__(attrname)
 
     def spectral_interpolate(self, spectral_grid,
                              suppress_smooth_warning=False,
@@ -854,7 +919,9 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
 
         if outdiff > 2 * indiff and not suppress_smooth_warning:
             warnings.warn("Input grid has too small a spacing. The data should "
-                          "be smoothed prior to resampling.")
+                          "be smoothed prior to resampling.",
+                          SmoothingWarning
+                         )
 
         newspec = np.empty([spectral_grid.size], dtype=self.dtype)
         newmask = np.empty([spectral_grid.size], dtype='bool')
@@ -923,7 +990,7 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         See `astropy.units.Quantity.to` for further details.
         """
 
-        return super(OneDSpectrum, self).to(unit, equivalencies, freq=None)
+        return super(BaseOneDSpectrum, self).to(unit, equivalencies, freq=None)
 
     def with_fill_value(self, fill_value):
         """
@@ -931,9 +998,14 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         """
         return self._new_spectrum_with(fill_value=fill_value)
 
+    @property
+    def _new_thing_with(self):
+        return self._new_spectrum_with
+
     def _new_spectrum_with(self, data=None, wcs=None, mask=None, meta=None,
                            fill_value=None, spectral_unit=None, unit=None,
-                           header=None, wcs_tolerance=None, **kwargs):
+                           header=None, wcs_tolerance=None,
+                           **kwargs):
 
         data = self._data if data is None else data
         if unit is None and hasattr(data, 'unit'):
@@ -975,30 +1047,138 @@ class OneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
                                   header=header or self._header,
                                   wcs_tolerance=wcs_tolerance or self._wcs_tolerance,
                                   **kwargs)
+
         spectrum._spectral_unit = spectral_unit
 
         return spectrum
 
-    def __getattribute__(self, attrname):
-        # This is a hack to handle dimensionality-reducing functions
-        # We want spectrum.max() to return a Quantity, not a spectrum
-        # Long-term, we really want `OneDSpectrum` to not inherit from
-        # `Quantity`, but for now this approach works.... we just have
-        # to add more functions to this list.
-        if attrname in ('min', 'max', 'std', 'mean', 'sum', 'cumsum',
-                        'nansum', 'ptp', 'var'):
-            return getattr(self.quantity, attrname)
-        else:
-            return super(OneDSpectrum, self).__getattribute__(attrname)
+
+class OneDSpectrum(BaseOneDSpectrum, BeamMixinClass):
+
+    def __new__(cls, value, beam=None, read_beam=False, **kwargs):
+        self = super(OneDSpectrum, cls).__new__(cls, value, **kwargs)
+
+        if beam is None:
+            if "beam" in self.meta:
+                beam = self.meta['beam']
+            elif read_beam:
+                beam = cube_utils.try_load_beam(self.header)
+                if beam is None:
+                    warnings.warn("Cannot load beam from header.",
+                                  BeamWarning
+                                  )
+
+        if beam is not None:
+            self.beam = beam
+            self.meta['beam'] = beam
+
+        return self
+
+    def _new_spectrum_with(self, **kwargs):
+        beam = kwargs.pop('beam', None)
+        if 'beam' in self._meta and beam is None:
+            beam = self.beam
+        out = super(OneDSpectrum, self)._new_spectrum_with(beam=beam, **kwargs)
+        return out
+
+    def with_beam(self, beam):
+        '''
+        Attach a new beam object to the OneDSpectrum.
+
+        Parameters
+        ----------
+        beam : `~radio_beam.Beam`
+            A new beam object.
+        '''
+
+        meta = self.meta.copy()
+        meta['beam'] = beam
+
+        return self._new_spectrum_with(beam=beam, meta=meta)
+
+
+class VaryingResolutionOneDSpectrum(BaseOneDSpectrum, MultiBeamMixinClass):
+
+    def __new__(cls, value, beams=None, read_beam=False, goodbeams_mask=None, **kwargs):
+        self = super(VaryingResolutionOneDSpectrum, cls).__new__(cls, value, **kwargs)
+        assert hasattr(self, '_fill_value')
+
+        if beams is None:
+            if "beams" in self.meta:
+                beams = self.meta['beams']
+            elif read_beam:
+                beams = cube_utils.try_load_beams(self.header)
+                if beams is None:
+                    warnings.warn("Cannot load beams table from header.",
+                                  BeamWarning
+                                  )
+
+        if beams is not None:
+            if isinstance(beams, BinTableHDU):
+                beam_data_table = beams.data
+            elif isinstance(beams, FITS_rec):
+                beam_data_table = beams
+            else:
+                beam_data_table = None
+
+            if beam_data_table is not None:
+                beams = Beams(major=u.Quantity(beam_data_table['BMAJ'], u.arcsec),
+                              minor=u.Quantity(beam_data_table['BMIN'], u.arcsec),
+                              pa=u.Quantity(beam_data_table['BPA'], u.deg),
+                              meta=[{key: row[key] for key in beam_data_table.names
+                                     if key not in ('BMAJ','BPA', 'BMIN')}
+                                    for row in beam_data_table],)
+            self.beams = beams
+            self.meta['beams'] = beams
+
+        if goodbeams_mask is not None:
+            self.goodbeams_mask = goodbeams_mask
+
+        return self
+
+    @property
+    def hdu(self):
+        warnings.warn("There are multiple beams for this spectrum that "
+                      "are being ignored when creating the HDU.",
+                      BeamWarning
+                     )
+        return super(VaryingResolutionOneDSpectrum, self).hdu
+
+    @property
+    def hdulist(self):
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            hdu = self.hdu
+
+        beamhdu = cube_utils.beams_to_bintable(self.beams)
+
+        return HDUList([hdu, beamhdu])
+
+    def _new_spectrum_with(self, **kwargs):
+        beams = kwargs.pop('beams', self.beams)
+        if beams is None:
+            beams = self.beams
+
+        VRODS = VaryingResolutionOneDSpectrum
+        out = super(VRODS, self)._new_spectrum_with(beams=beams,
+                                                    **kwargs)
+        return out
 
     def __array_finalize__(self, obj):
-        #from astropy import log
-        #log.debug("in OneDSpectrum, Finalizing self={0}{1} obj={2}{3}"
-        #          .format(self, type(self), obj, type(obj)))
-        self._fill_value = getattr(obj, '_fill_value', np.nan)
-        self._data = self.view(np.ndarray)
-        self._wcs_tolerance = getattr(obj, '_wcs_tolerance', 0.0)
-        super(OneDSpectrum, self).__array_finalize__(obj)
+        super(VaryingResolutionOneDSpectrum, self).__array_finalize__(obj)
 
-class VaryingResolutionOneDSpectrum(OneDSpectrum, MultiBeamMixinClass):
-    pass
+        self._beams = getattr(obj, '_beams', None)
+        if getattr(obj, 'goodbeams_mask', None) is not None:
+            # do NOT use the setter here, because we sometimes need to write
+            # intermediate size-mismatch things that later get fixed, e.g., in
+            # __getitem__ below
+            self._goodbeams_mask = getattr(obj, 'goodbeams_mask', None)
+
+    def __getitem__(self, key):
+        new_qty = super(VaryingResolutionOneDSpectrum, self).__getitem__(key)
+
+        # use the goodbeams_mask setter here because it checks size
+        new_qty.goodbeams_mask = self.goodbeams_mask[key]
+        new_qty.beams = self.unmasked_beams[key]
+
+        return new_qty
