@@ -1,6 +1,8 @@
 from __future__ import print_function, absolute_import, division
 
 import abc
+import warnings
+import tempfile
 
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
@@ -10,6 +12,7 @@ from astropy.io import fits
 from astropy.extern.six.moves import zip
 
 from . import wcs_utils
+from .utils import WCSWarning
 
 
 __all__ = ['MaskBase', 'InvertedMask', 'CompositeMask', 'BooleanArrayMask',
@@ -112,6 +115,22 @@ class MaskBase(object):
         self._validate_wcs(data, wcs, **kwargs)
         return self._include(data=data, wcs=wcs, view=view)
 
+    # Commented out, but left as a possibility, because including this didn't fix any
+    # of the problems we encountered with matplotlib plotting
+    def view(self, view=()):
+        """
+        Compatibility tool: if a numpy.ma.ufunc is run on the mask, it will try
+        to grab a view of the mask, which needs to appear to numpy as a true
+        array.  This can be important for, e.g., plotting.
+
+        Numpy's convention is that masked=True means "masked out"
+
+        .. note::
+            I don't know if there are broader concerns or consequences from
+            including this 'view' tool here.
+        """
+        return self.exclude(view=view)
+
     def _validate_wcs(self, new_data=None, new_wcs=None, **kwargs):
         """
         This method can be overridden in cases where the data and WCS have to
@@ -165,7 +184,8 @@ class MaskBase(object):
         """
         return data[view][self.include(data=data, wcs=wcs, view=view)]
 
-    def _filled(self, data, wcs=None, fill=np.nan, view=(), **kwargs):
+    def _filled(self, data, wcs=None, fill=np.nan, view=(), use_memmap=False,
+                **kwargs):
         """
         Replace the excluded elements of *array* with *fill*.
 
@@ -177,6 +197,8 @@ class MaskBase(object):
             Replacement value
         view : tuple, optional
             Any slicing to apply to the data before flattening
+        use_memmap : bool
+            Use a memory map to store the output data?
 
         Returns
         -------
@@ -191,7 +213,15 @@ class MaskBase(object):
         # Must convert to floating point, but should not change from inherited
         # type otherwise
         dt = np.find_common_type([data.dtype], [np.float])
-        sliced_data = data[view].astype(dt)
+
+        if use_memmap and data.size > 0:
+            ntf = tempfile.NamedTemporaryFile()
+            sliced_data = np.memmap(ntf, mode='w+', shape=data[view].shape,
+                                    dtype=dt)
+            sliced_data[:] = data[view]
+        else:
+            sliced_data = data[view].astype(dt)
+
         ex = self.exclude(data=data, wcs=wcs, view=view, **kwargs)
         sliced_data[ex] = fill
         return sliced_data
@@ -485,11 +515,19 @@ class BooleanArrayMask(MaskBase):
             raise ValueError("data shape cannot be broadcast to match mask shape")
         if new_wcs is not None:
             if new_wcs not in self._wcs_whitelist:
-                if not wcs_utils.check_equality(new_wcs, self._wcs,
-                                                warn_missing=True,
-                                                **kwargs):
-                    raise ValueError("WCS does not match mask WCS")
-                else:
+                try:
+                    if not wcs_utils.check_equality(new_wcs, self._wcs,
+                                                    warn_missing=True,
+                                                    **kwargs):
+                        raise ValueError("WCS does not match mask WCS")
+                    else:
+                        self._wcs_whitelist.add(new_wcs)
+                except InconsistentAxisTypesError:
+                    warnings.warn("Inconsistent axis type encountered; WCS is "
+                                  "invalid and therefore will not be checked "
+                                  "against other WCSes.",
+                                  WCSWarning
+                                  )
                     self._wcs_whitelist.add(new_wcs)
 
     def _include(self, data=None, wcs=None, view=()):
