@@ -1,4 +1,6 @@
+import sys
 import pytest
+import tempfile
 import numpy as np
 
 from astropy import units as u
@@ -7,25 +9,25 @@ from astropy.wcs import WCS
 from astropy import wcs
 from astropy.io import fits
 
+try:
+    import tracemalloc
+    tracemallocOK = True
+except ImportError:
+    tracemallocOK = False
+
+# The comparison of Quantities in test_memory_usage
+# fail with older versions of numpy
+from distutils.version import LooseVersion
+
+NPY_VERSION_CHECK = LooseVersion(np.version.version) >= "1.13"
+
 from radio_beam import beam, Beam
 
 from .. import SpectralCube
 from ..utils import WCSCelestialError
 from .test_spectral_cube import cube_and_raw
 from .test_projection import load_projection
-from . import path
-
-try:
-    import reproject
-    REPROJECT_INSTALLED = True
-except ImportError:
-    REPROJECT_INSTALLED = False
-
-try:
-    import joblib
-    JOBLIB_INSTALLED = True
-except ImportError:
-    JOBLIB_INSTALLED = False
+from . import path, utilities
 
 
 def test_convolution():
@@ -63,7 +65,7 @@ def test_beams_convolution():
 
     pixscale = wcs.utils.proj_plane_pixel_area(cube.wcs.celestial)**0.5*u.deg
 
-    for ii,bm in enumerate(cube.beams):
+    for ii, bm in enumerate(cube.beams):
         expected = target_beam.deconvolve(bm).as_kernel(pixscale, x_size=5,
                                                         y_size=5)
         expected.normalize()
@@ -88,8 +90,11 @@ def test_beams_convolution_equal():
     np.testing.assert_almost_equal(cube.filled_data[0].value,
                                    conv_cube.filled_data[0].value)
 
-@pytest.mark.skipif('not REPROJECT_INSTALLED')
-def test_reproject():
+
+@pytest.mark.parametrize('use_memmap', (True, False))
+def test_reproject(use_memmap):
+
+    pytest.importorskip('reproject')
 
     cube, data = cube_and_raw('adv.fits')
 
@@ -105,10 +110,11 @@ def test_reproject():
     header_out['NAXIS3'] = cube.shape[0]
     header_out.update(wcs_out.to_header())
 
-    result = cube.reproject(header_out)
+    result = cube.reproject(header_out, use_memmap=use_memmap)
 
     assert result.shape == (cube.shape[0], 5, 4)
 
+
 def test_spectral_smooth():
 
     cube, data = cube_and_raw('522_delta.fits')
@@ -128,12 +134,11 @@ def test_spectral_smooth():
                                    4)
 
 
-# TODO: uncomment this when we figure out how to make it work
-@pytest.mark.skipif('not JOBLIB_INSTALLED')
 def test_spectral_smooth_4cores():
 
-    cube, data = cube_and_raw('522_delta.fits')
+    pytest.importorskip('joblib')
 
+    cube, data = cube_and_raw('522_delta.fits')
 
     result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), num_cores=4, use_memmap=True)
 
@@ -160,7 +165,6 @@ def test_spectral_smooth_4cores():
                                    4)
 
 
-
 def test_spectral_smooth_fail():
 
     cube, data = cube_and_raw('522_delta_beams.fits')
@@ -205,7 +209,6 @@ def test_spectral_interpolate_with_fillvalue():
                                    np.ones(4)*42)
 
 
-
 def test_spectral_interpolate_fail():
 
     cube, data = cube_and_raw('522_delta_beams.fits')
@@ -242,7 +245,7 @@ def test_spectral_interpolate_with_mask():
 
     # The output makes CDELT3 > 0 (reversed spectral axis) so the masked
     # portion are the final 2 channels.
-    np.testing.assert_almost_equal(result[:,0, 0].value,
+    np.testing.assert_almost_equal(result[:, 0, 0].value,
                                    [0.0, 0.5, np.NaN, np.NaN])
 
     assert cube.wcs.wcs.compare(orig_wcs.wcs)
@@ -262,7 +265,6 @@ def test_spectral_interpolate_reversed():
     np.testing.assert_almost_equal(sg.value, result.spectral_axis.value)
 
 
-
 def test_convolution_2D():
 
     proj, hdu = load_projection("55_delta.fits")
@@ -300,9 +302,10 @@ def test_nocelestial_convolution_2D_fail():
     assert exc.value.args[0] == ("WCS does not contain two spatial axes.")
 
 
-@pytest.mark.skipif('not REPROJECT_INSTALLED')
 def test_reproject_2D():
 
+    pytest.importorskip('reproject')
+
     proj, hdu = load_projection("55.fits")
 
     wcs_in = WCS(proj.header)
@@ -322,9 +325,10 @@ def test_reproject_2D():
     assert result.beam == proj.beam
 
 
-@pytest.mark.skipif('not REPROJECT_INSTALLED')
 def test_nocelestial_reproject_2D_fail():
 
+    pytest.importorskip('reproject')
+
     cube, data = cube_and_raw('255_delta.fits')
 
     proj = cube.moment0(axis=1)
@@ -333,3 +337,146 @@ def test_nocelestial_reproject_2D_fail():
         proj.reproject(cube.header)
 
     assert exc.value.args[0] == ("WCS does not contain two spatial axes.")
+
+
+@pytest.mark.parametrize('use_memmap', (True,False))
+def test_downsample(use_memmap):
+    cube, data = cube_and_raw('255.fits')
+
+    dscube = cube.downsample_axis(factor=2, axis=0, use_memmap=use_memmap)
+
+    expected = data.mean(axis=0)
+
+    np.testing.assert_almost_equal(expected[None,:,:],
+                                   dscube.filled_data[:].value)
+
+    dscube = cube.downsample_axis(factor=2, axis=1, use_memmap=use_memmap)
+
+    expected = np.array([data[:,:2,:].mean(axis=1),
+                         data[:,2:4,:].mean(axis=1),
+                         data[:,4:,:].mean(axis=1), # just data[:,4,:]
+                        ]).swapaxes(0,1)
+    assert expected.shape == (2,3,5)
+    assert dscube.shape == (2,3,5)
+
+    np.testing.assert_almost_equal(expected,
+                                   dscube.filled_data[:].value)
+
+    dscube = cube.downsample_axis(factor=2, axis=1, truncate=True,
+                                  use_memmap=use_memmap)
+
+    expected = np.array([data[:,:2,:].mean(axis=1),
+                         data[:,2:4,:].mean(axis=1),
+                        ]).swapaxes(0,1)
+
+    np.testing.assert_almost_equal(expected,
+                                   dscube.filled_data[:].value)
+
+
+
+@pytest.mark.parametrize('use_memmap', (True,False))
+def test_downsample_wcs(use_memmap):
+    cube, data = cube_and_raw('255.fits')
+
+    dscube = (cube
+              .downsample_axis(factor=2, axis=1, use_memmap=use_memmap)
+              .downsample_axis(factor=2, axis=2, use_memmap=use_memmap))
+
+    # pixel [0,0] in the new cube should have coordinate [1,1] in the old cube
+
+    lonnew, latnew = dscube.wcs.celestial.wcs_pix2world(0, 0, 0)
+    xpixold_ypixold = np.array(cube.wcs.celestial.wcs_world2pix(lonnew, latnew, 0))
+
+    np.testing.assert_almost_equal(xpixold_ypixold, (0.5, 0.5))
+
+    # the center of the bottom-left pixel, in FITS coordinates, in the
+    # original frame will now be at -0.25, -0.25 in the new frame
+    lonold, latold = cube.wcs.celestial.wcs_pix2world(1, 1, 1)
+    xpixnew_ypixnew = np.array(dscube.wcs.celestial.wcs_world2pix(lonold, latold, 1))
+
+    np.testing.assert_almost_equal(xpixnew_ypixnew, (0.75, 0.75))
+
+@pytest.mark.skipif('not tracemallocOK or (sys.version_info.major==3 and sys.version_info.minor<6) or not NPY_VERSION_CHECK')
+def test_reproject_3D_memory():
+
+    pytest.importorskip('reproject')
+
+    tracemalloc.start()
+
+    snap1 = tracemalloc.take_snapshot()
+
+    # create a 64 MB cube
+    cube,_ = utilities.generate_gaussian_cube(shape=[200,200,200])
+    sz = _.dtype.itemsize
+
+    # check that cube is loaded into memory
+    snap2 = tracemalloc.take_snapshot()
+    diff = snap2.compare_to(snap1, 'lineno')
+    diffvals = np.array([dd.size_diff for dd in diff])
+    # at this point, the generated cube should still exist in memory
+    assert diffvals.max()*u.B >= 200**3*sz*u.B
+
+    wcs_in = cube.wcs
+    wcs_out = wcs_in.deepcopy()
+    wcs_out.wcs.ctype = ['GLON-SIN', 'GLAT-SIN', cube.wcs.wcs.ctype[2]]
+    wcs_out.wcs.crval = [0.001, 0.001, cube.wcs.wcs.crval[2]]
+    wcs_out.wcs.crpix = [2., 2., cube.wcs.wcs.crpix[2]]
+
+    header_out = (wcs_out.to_header())
+    header_out['NAXIS'] = 3
+    header_out['NAXIS1'] = int(cube.shape[2]/2)
+    header_out['NAXIS2'] = int(cube.shape[1]/2)
+    header_out['NAXIS3'] = cube.shape[0]
+
+    # First the unfilled reprojection test: new memory is allocated for
+    # `result`, but nowhere else
+    result = cube.reproject(header_out, filled=False)
+
+    snap3 = tracemalloc.take_snapshot()
+    diff = snap3.compare_to(snap2, 'lineno')
+    diffvals = np.array([dd.size_diff for dd in diff])
+    # result should have the same size as the input data, except smaller in two dims
+    # make sure that's all that's allocated
+    assert diffvals.max()*u.B >= 200*100**2*sz*u.B
+    assert diffvals.max()*u.B < 200*110**2*sz*u.B
+
+    # without masking the cube, nothing should change
+    result = cube.reproject(header_out, filled=True)
+
+    snap4 = tracemalloc.take_snapshot()
+    diff = snap4.compare_to(snap3, 'lineno')
+    diffvals = np.array([dd.size_diff for dd in diff])
+    assert diffvals.max()*u.B <= 1*u.MB
+
+    assert result.wcs.wcs.crval[0] == 0.001
+    assert result.wcs.wcs.crpix[0] == 2.
+
+
+    # masking the cube will force the fill to create a new in-memory copy
+    mcube = cube.with_mask(cube > 0.1*cube.unit)
+    # `_is_huge` would trigger a use_memmap
+    assert not mcube._is_huge
+    assert mcube.mask.any()
+
+    # take a new snapshot because we're not testing the mask creation
+    snap5 = tracemalloc.take_snapshot()
+    tracemalloc.stop()
+    tracemalloc.start() # stop/start so we can check peak mem use from here
+    current_b4, peak_b4 = tracemalloc.get_traced_memory()
+    result = mcube.reproject(header_out, filled=True)
+    current_aftr, peak_aftr = tracemalloc.get_traced_memory()
+
+
+    snap6 = tracemalloc.take_snapshot()
+    diff = snap6.compare_to(snap5, 'lineno')
+    diffvals = np.array([dd.size_diff for dd in diff])
+    # a duplicate of the cube should have been created by filling masked vals
+    # (this should be near-exact since 'result' should occupy exactly the
+    # same amount of memory)
+    assert diffvals.max()*u.B <= 1*u.MB #>= 200**3*sz*u.B
+    # the peak memory usage *during* reprojection will have that duplicate,
+    # but the memory gets cleaned up afterward
+    assert (peak_aftr-peak_b4)*u.B >= (200**3*sz*u.B + 200*100**2*sz*u.B)
+
+    assert result.wcs.wcs.crval[0] == 0.001
+    assert result.wcs.wcs.crpix[0] == 2.
