@@ -20,7 +20,9 @@ import numpy as np
 
 from .. import (SpectralCube, VaryingResolutionSpectralCube, BooleanArrayMask,
                 FunctionMask, LazyMask, CompositeMask)
-from ..spectral_cube import OneDSpectrum, Projection, VaryingResolutionOneDSpectrum
+from ..spectral_cube import (OneDSpectrum, Projection,
+                             VaryingResolutionOneDSpectrum,
+                             LowerDimensionalObject)
 from ..np_compat import allbadtonan
 from .. import spectral_axis
 from .. import base_class
@@ -35,17 +37,6 @@ warnings.simplefilter('error', utils.UnsupportedIterationStrategyWarning)
 warnings.simplefilter('error', utils.NotImplementedWarning)
 warnings.simplefilter('error', utils.WCSMismatchWarning)
 
-try:
-    import joblib
-    JOBLIB_INSTALLED = True
-except ImportError:
-    JOBLIB_INSTALLED = False
-
-try:
-    import scipy.ndimage
-    SCIPYOK = True
-except ImportError:
-    SCIPYOK = False
 
 try:
     import yt
@@ -55,13 +46,7 @@ except ImportError:
     YT_INSTALLED = False
     YT_LT_301 = False
 
-try:
-    import bottleneck
-    BOTTLENECK_INSTALLED = True
-except ImportError:
-    BOTTLENECK_INSTALLED = False
-
-from radio_beam import Beam
+from radio_beam import Beam, Beams
 
 NUMPY_LT_19 = LooseVersion(np.__version__) < LooseVersion('1.9.0')
 
@@ -665,7 +650,6 @@ SpectralCube with shape=(4, 3, 2) and unit=Jy:
         """.strip()
 
 
-@pytest.mark.xfail
 @pytest.mark.skipif('not YT_INSTALLED')
 class TestYt():
     def setup_method(self, method):
@@ -692,9 +676,7 @@ class TestYt():
                         ds2.domain_width.value*np.array([1,1,1.0/self.spectral_factor]))
         assert_allclose(ds1.domain_width.value, ds3.domain_width.value)
         assert self.nprocs == len(ds3.index.grids)
-        assert ds1.spec_cube
-        assert ds2.spec_cube
-        assert ds3.spec_cube
+
         ds1.index
         ds2.index
         ds3.index
@@ -860,7 +842,6 @@ def test_preserve_spectral_unit():
     assert new_cube.spectral_axis.unit is u.GHz
 
 
-@pytest.mark.skipif('not BOTTLENECK_INSTALLED')
 def test_endians():
     """
     Test that the endianness checking returns something in Native form
@@ -872,6 +853,7 @@ def test_endians():
     little-endian to native in the dtype parameter; I need a workaround for
     this.
     """
+    pytest.importorskip('bottleneck')
     big = np.array([[[1],[2]]], dtype='>f4')
     lil = np.array([[[1],[2]]], dtype='<f4')
     mywcs = WCS(naxis=3)
@@ -1129,6 +1111,34 @@ def test_preserves_header_values():
     assert proj.header['OBJECT'] == 'TestName'
     assert proj.hdu.header['OBJECT'] == 'TestName'
 
+def test_preserves_header_meta_values():
+    # Check that additional parameters in meta are preserved
+
+    cube, data = cube_and_raw('advs.fits')
+
+    cube.meta['foo'] = 'bar'
+
+    assert cube.header['FOO'] == 'bar'
+
+    # check that long keywords are also preserved
+    cube.meta['too_long_keyword'] = 'too_long_information'
+
+    assert 'too_long_keyword=too_long_information' in cube.header['COMMENT']
+
+    # Checks that the header is preserved when passed to LDOs
+    for ldo in (cube.sum(axis=0, how='auto'), cube[:,0,0]):
+        assert isinstance(ldo, LowerDimensionalObject)
+        assert ldo.header['FOO'] == 'bar'
+        assert ldo.hdu.header['FOO'] == 'bar'
+
+        # make sure that the meta preservation works on the LDOs themselves too
+        ldo.meta['bar'] = 'foo'
+        assert ldo.header['BAR'] == 'foo'
+
+        assert 'too_long_keyword=too_long_information' in ldo.header['COMMENT']
+
+
+
 @pytest.mark.parametrize('func',('sum','std','max','min','mean'))
 def test_oned_numpy(func):
     # Check that a numpy function returns an appropriate spectrum
@@ -1167,7 +1177,7 @@ def test_oned_slice_beams():
     cube._unit = u.K
 
     spec = cube[:,0,0]
-    assert isinstance(spec, OneDSpectrum)
+    assert isinstance(spec, VaryingResolutionOneDSpectrum)
     # data has a redundant 1st axis
     np.testing.assert_equal(spec.value, data[:,0,0,0])
     assert cube.unit == spec.unit
@@ -1183,6 +1193,14 @@ def test_subcube_slab_beams():
 
     assert all(slcube.hdulist[1].data['CHAN'] == np.arange(slcube.shape[0]))
 
+    try:
+        # Make sure Beams has been sliced correctly
+        assert all(cube.beams[1:] == slcube.beams)
+    except TypeError:
+        # in 69eac9241220d3552c06b173944cb7cdebeb47ef, radio_beam switched to
+        # returning a single value
+        assert cube.beams[1:] == slcube.beams
+
 # collapsing to one dimension raywise doesn't make sense and is therefore
 # not supported.
 @pytest.mark.parametrize('how', ('auto', 'cube', 'slice'))
@@ -1211,7 +1229,7 @@ def test_oned_collapse_beams():
     cube._unit = u.K
 
     spec = cube.mean(axis=(1,2))
-    assert isinstance(spec, OneDSpectrum)
+    assert isinstance(spec, VaryingResolutionOneDSpectrum)
     # data has a redundant 1st axis
     np.testing.assert_equal(spec.value, data.mean(axis=(1,2,3)))
     assert cube.unit == spec.unit
@@ -1303,6 +1321,36 @@ def test_beam_custom():
     assert newcube2.meta['beam'] == newbeam
 
 
+def test_multibeam_custom():
+
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    # Make a new set of beams that differs from the original.
+    new_beams = Beams([1.] * cube.shape[0] * u.deg)
+
+    # Attach the beam
+    newcube = cube.with_beams(new_beams)
+
+    try:
+        assert all(new_beams == newcube.beams)
+    except TypeError:
+        # in 69eac9241220d3552c06b173944cb7cdebeb47ef, radio_beam switched to
+        # returning a single value
+        assert new_beams == newcube.beams
+
+
+@pytest.mark.xfail(raises=ValueError, strict=True)
+def test_multibeam_custom_wrongshape():
+
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    # Make a new set of beams that differs from the original.
+    new_beams = Beams([1.] * cube.shape[0] * u.deg)
+
+    # Attach the beam
+    cube.with_beams(new_beams[:1])
+
+
 def test_multibeam_slice():
 
     cube, data = cube_and_raw('vda_beams.fits')
@@ -1565,9 +1613,9 @@ def test_mask_bad_beams():
     cube, data = cube_and_raw('vda_beams.fits')
 
     # make sure all of the beams are initially good (finite)
-    assert np.all(cube._goodbeams_mask)
+    assert np.all(cube.goodbeams_mask)
     # make sure cropping the cube maintains the mask
-    assert np.all(cube[:3]._goodbeams_mask)
+    assert np.all(cube[:3].goodbeams_mask)
 
     # middle two beams have same area
     masked_cube = cube.mask_out_bad_beams(0.01,
@@ -1576,7 +1624,7 @@ def test_mask_bad_beams():
                                                               60*u.deg))
 
     assert np.all(masked_cube.mask.include()[:,0,0] == [False,False,True,False])
-    assert np.all(masked_cube._goodbeams_mask == [False,False,True,False])
+    assert np.all(masked_cube.goodbeams_mask == [False,False,True,False])
 
     mean = masked_cube.mean(axis=0)
     assert np.all(mean == cube[2,:,:])
@@ -1586,7 +1634,14 @@ def test_mask_bad_beams():
 
     mean2 = masked_cube2.mean(axis=0)
     assert np.all(mean2 == (cube[2,:,:]+cube[1,:,:])/2)
-    assert np.all(masked_cube2._goodbeams_mask == [False,True,True,False])
+    assert np.all(masked_cube2.goodbeams_mask == [False,True,True,False])
+
+
+def test_convolve_to():
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    convolved = cube.convolve_to(Beam(0.5*u.arcsec))
+
 
 def test_convolve_to_with_bad_beams():
     cube, data = cube_and_raw('vda_beams.fits')
@@ -1647,6 +1702,30 @@ def test_mad_std():
 
         np.testing.assert_almost_equal(mcube.mad_std(axis=0).value, result2)
 
+def test_mad_std_params():
+    cube, data = cube_and_raw('adv.fits')
+
+    # mad_std run manually on data
+    result = np.array([[0.15509701,  0.45763670],
+                       [0.55907956,  0.42932451],
+                       [0.48819454,  0.25499305]])
+
+    np.testing.assert_almost_equal(cube.mad_std(axis=0, how='cube').value, result)
+    np.testing.assert_almost_equal(cube.mad_std(axis=0, how='ray').value, result)
+
+    with pytest.raises(NotImplementedError) as exc:
+        cube.mad_std(axis=0, how='slice')
+
+    with pytest.raises(NotImplementedError) as exc:
+        cube.mad_std(axis=1, how='slice')
+
+    with pytest.raises(NotImplementedError) as exc:
+        cube.mad_std(axis=(1,2), how='ray')
+
+    # stats.mad_std(data, axis=(1,2))
+    np.testing.assert_almost_equal(cube.mad_std(axis=0, how='ray').value, result)
+
+
 def test_caching():
 
     cube, data = cube_and_raw('adv.fits')
@@ -1687,6 +1766,21 @@ def test_spatial_smooth_g2d():
 
     np.testing.assert_almost_equal(cube_g2d[2].value, result2)
 
+def test_spatial_smooth_preserves_unit():
+    """
+    Regression test for issue527
+    """
+    cube, data = cube_and_raw('adv.fits')
+    cube._unit = u.K
+
+    #
+    # Guassian 2D smoothing test
+    #
+    g2d = Gaussian2DKernel(3)
+    cube_g2d = cube.spatial_smooth(g2d)
+
+    assert cube_g2d.unit == u.K
+
 def test_spatial_smooth_t2d():
     cube, data = cube_and_raw('adv.fits')
 
@@ -1710,8 +1804,11 @@ def test_spatial_smooth_t2d():
 
     np.testing.assert_almost_equal(cube_t2d[2].value, result2)
 
-@pytest.mark.skipif('not SCIPYOK')
+
 def test_spatial_smooth_median():
+
+    pytest.importorskip('scipy.ndimage')
+
     cube, data = cube_and_raw('adv.fits')
 
     cube_median = cube.spatial_smooth_median(3)
@@ -1730,8 +1827,11 @@ def test_spatial_smooth_median():
 
     np.testing.assert_almost_equal(cube_median[2].value, result2)
 
-@pytest.mark.skipif('not SCIPYOK')
+
 def test_spectral_smooth_median():
+
+    pytest.importorskip('scipy.ndimage')
+
     cube, data = cube_and_raw('adv.fits')
 
     cube_spectral_median = cube.spectral_smooth_median(3)
@@ -1741,9 +1841,12 @@ def test_spectral_smooth_median():
 
     np.testing.assert_almost_equal(cube_spectral_median[:,1,1].value, result)
 
-@pytest.mark.skipif('not SCIPYOK')
-@pytest.mark.skipif('not JOBLIB_INSTALLED')
+
 def test_spectral_smooth_median_4cores():
+
+    pytest.importorskip('joblib')
+    pytest.importorskip('scipy.ndimage')
+
     cube, data = cube_and_raw('adv.fits')
 
     cube_spectral_median = cube.spectral_smooth_median(3, num_cores=4)
@@ -1778,12 +1881,11 @@ def test_varyres_spectra():
     assert isinstance(sp, VaryingResolutionOneDSpectrum)
     assert hasattr(sp, 'beams')
 
+
 def test_median_2axis():
     """
-    As of this writing the bottleneck.nanmedian did not accept an axis that is a tuple/list so this test
-    is to make sure that is properly taken into account.
-
-    :return:
+    As of this writing the bottleneck.nanmedian did not accept an axis that is a
+    tuple/list so this test is to make sure that is properly taken into account.
     """
     cube, data = cube_and_raw('adv.fits')
 
@@ -1793,3 +1895,34 @@ def test_median_2axis():
     result0 = np.array([0.83498009, 0.2606566 , 0.37271531, 0.48548023])
 
     np.testing.assert_almost_equal(cube_median.value, result0)
+
+
+def test_varyres_mask():
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    # mask out two beams
+    goodbeams = cube.identify_bad_beams(0.5)
+    assert all(goodbeams == np.array([False, True, True, False]))
+
+    mcube = cube.mask_out_bad_beams(0.5)
+    assert hasattr(mcube, '_goodbeams_mask')
+    assert all(mcube.goodbeams_mask == goodbeams)
+    assert len(mcube.beams) == 2
+
+    sp_masked = mcube[:,0,0]
+
+    assert hasattr(sp_masked, '_goodbeams_mask')
+    assert all(sp_masked.goodbeams_mask == goodbeams)
+    assert len(sp_masked.beams) == 2
+
+    try:
+        assert mcube.unmasked_beams == cube.beams
+    except ValueError:
+        # older versions of beams
+        assert np.all(mcube.unmasked_beams == cube.beams)
+
+    try:
+        # check that slicing works too
+        assert mcube[:5].unmasked_beams == cube[:5].beams
+    except ValueError:
+        assert np.all(mcube[:5].unmasked_beams == cube[:5].beams)
