@@ -32,31 +32,66 @@ def warn_slow(function):
                              "pixels), so by default we disable this operation. "
                              "To enable the operation, set "
                              "`cube.allow_huge_operations=True` and try again."
-                             .format(str(function), self.size))
+                             .format(str(function), self.size,))
         elif warn_how and not self._is_huge:
             # TODO: add check for whether cube has been loaded into memory
             warnings.warn("This function ({0}) requires loading the entire cube into "
-                          "memory and may therefore be slow.".format(str(function)))
+                          "memory and may therefore be slow.".format(str(function)),
+                          PossiblySlowWarning
+                         )
         return function(self, *args, **kwargs)
     return wrapper
 
-class UnsupportedIterationStrategyWarning(AstropyUserWarning):
+class SpectralCubeWarning(AstropyUserWarning):
     pass
 
-class VarianceWarning(AstropyUserWarning):
+class UnsupportedIterationStrategyWarning(SpectralCubeWarning):
     pass
 
-class SliceWarning(AstropyUserWarning):
+class VarianceWarning(SpectralCubeWarning):
     pass
 
-class BeamAverageWarning(AstropyUserWarning):
+class SliceWarning(SpectralCubeWarning):
+    pass
+
+class BeamAverageWarning(SpectralCubeWarning):
+    pass
+
+class BeamWarning(SpectralCubeWarning):
     pass
 
 class WCSCelestialError(Exception):
     pass
 
-class WCSMismatchWarning(AstropyUserWarning):
+class WCSMismatchWarning(SpectralCubeWarning):
     pass
 
-class NotImplementedWarning(AstropyUserWarning):
+class NotImplementedWarning(SpectralCubeWarning):
+    pass
+
+class StokesWarning(SpectralCubeWarning):
+    pass
+
+class ExperimentalImplementationWarning(SpectralCubeWarning):
+    pass
+
+class PossiblySlowWarning(SpectralCubeWarning):
+    pass
+
+class SmoothingWarning(SpectralCubeWarning):
+    pass
+
+class NonFiniteBeamsWarning(SpectralCubeWarning):
+    pass
+
+class WCSWarning(SpectralCubeWarning):
+    pass
+
+class FITSWarning(SpectralCubeWarning):
+    pass
+
+class BadVelocitiesWarning(SpectralCubeWarning):
+    pass
+
+class FITSReadError(Exception):
     pass
